package com.nokia.finance.tasks.jobs.car.procedure;

import com.nokia.finance.tasks.common.exception.MyRuntimeException;
import com.nokia.finance.tasks.dao.car.CarProcedureDao;
import com.nokia.finance.tasks.dao.gdc.car.GdcCarProcedureDao;
import com.nokia.finance.tasks.enums.DataStageEnum;
import com.nokia.finance.tasks.pojo.po.common.DataLogPo;
import com.nokia.finance.tasks.service.common.DataLogServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.CompletableFuture;

/**
 * 车辆基本信息月数据加工定时任务
 */
@Slf4j
@Service
public class CarBaseDataMonthProcJob {
    private final CarProcedureDao carProcedureDao;
    private final GdcCarProcedureDao gdcCarProcedureDao;
    private final DataLogServiceImpl dataLogService;

    public CarBaseDataMonthProcJob(CarProcedureDao carProcedureDao, GdcCarProcedureDao gdcCarProcedureDao,
                                   DataLogServiceImpl dataLogService) {
        this.carProcedureDao = carProcedureDao;
        this.gdcCarProcedureDao = gdcCarProcedureDao;
        this.dataLogService = dataLogService;
    }

    /**
     * 执行任务
     */
    @Scheduled(cron = "0 0 0 7 * ?")
    @Transactional(timeout = 60, rollbackFor = Exception.class)
    public void runJob() {
        log.info("执行车辆基本信息月数据加工定时任务");
        List<DataLogPo> dataLogPoList = new ArrayList<>();
        DataLogPo dataLogPo = new DataLogPo();
        dataLogPo.setDataName("车辆基本信息月数据加工");
        dataLogPo.setStage(DataStageEnum.JG.value);
        dataLogPo.setOperationName("车辆基本信息月数据加工");
        dataLogPo.setObject("car.car_base_data_month");
        dataLogPoList.add(dataLogPo);
        try {
            LocalDate localDateNow = LocalDate.now();
            LocalDate localDateEnd = localDateNow.minusMonths(1);
            Integer endYearMonth = Integer.valueOf(localDateEnd.format(DateTimeFormatter.ofPattern("yyyyMM")));
            int update1 = carProcedureDao.insertCarBaseDataMonth(endYearMonth);
            int update2 = carProcedureDao.updateCarBaseDataMonthChange(endYearMonth);
            int update3 = gdcCarProcedureDao.insertCarInfo(endYearMonth);
            gdcCarProcedureDao.insertCarScrap(endYearMonth);
            if (update1 == 0 || update2 == 0 || update3 == 0) {
                throw new MyRuntimeException("车辆基本信息月数据加工定时任务失败");
            }
            dataLogPo.setDataCount(update1);
        } catch (Exception e) {
            log.error(e.toString(), e);
            dataLogPo.setStatus(0);
            dataLogPo.setExceptionInfo(e.toString());
            throw new MyRuntimeException("车辆基本信息月数据加工定时任务失败");
        } finally {
            CompletableFuture.runAsync(() -> dataLogService.saveBatch(dataLogPoList));
        }
    }
}
