package com.nokia.finance.tasks.jobs.car.strategy;

import com.nokia.finance.tasks.common.exception.MyRuntimeException;
import com.nokia.finance.tasks.config.JobConfig;
import com.nokia.finance.tasks.dao.car.CarStrategyDao;
import com.nokia.finance.tasks.enums.DataStageEnum;
import com.nokia.finance.tasks.pojo.po.common.DataLogPo;
import com.nokia.finance.tasks.service.common.DataLogServiceImpl;
import com.nokia.finance.tasks.utils.FileUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.csv.CSVFormat;
import org.apache.commons.csv.CSVPrinter;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.io.OutputStreamWriter;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.CompletableFuture;

/**
 * 高修理车辆策略定时任务
 */
@Slf4j
@Service
public class CarHighRepairJob {
    private final JobConfig jobConfig;
    private final CarStrategyDao carStrategyDao;
    private final DataLogServiceImpl dataLogService;

    CarHighRepairJob(JobConfig jobConfig, CarStrategyDao carStrategyDao, DataLogServiceImpl dataLogService) {
        this.jobConfig = jobConfig;
        this.carStrategyDao = carStrategyDao;
        this.dataLogService = dataLogService;
    }
    /**
     * 执行任务
     */
    @Scheduled(cron = "0 6 0 15 * ?")
    @Transactional(timeout = 60, rollbackFor = Exception.class)
    public void runJob() {
        log.info("执行高修理车辆策略定时任务");
        List<DataLogPo> dataLogPoList = new ArrayList<>();
        DataLogPo dataLogPo = new DataLogPo();
        dataLogPo.setDataName("高修理车辆策略");
        dataLogPo.setStage(DataStageEnum.JG.value);
        dataLogPo.setOperationName("生成高修理车辆策略");
        dataLogPoList.add(dataLogPo);
        try {
            LocalDate localDateNow = LocalDate.now();
            LocalDate localDateEnd = localDateNow.minusMonths(1);
            LocalDate localDateStart = localDateEnd.withDayOfYear(1);
            Integer endYearMonth = Integer.valueOf(localDateEnd.format(DateTimeFormatter.ofPattern("yyyyMM")));
            Integer startYearMonth = Integer.valueOf(localDateStart.format(DateTimeFormatter.ofPattern("yyyyMM")));
            int updateCount = carStrategyDao.insertCarHighRepair(startYearMonth, endYearMonth);
            if (updateCount == 0) {
                throw new MyRuntimeException("插入高修理车辆策略失败");
            }
            List<LinkedHashMap<String, Object>> list = carStrategyDao.getCarHighRepair(endYearMonth);
            if (CollectionUtils.isEmpty(list)) {
                throw new MyRuntimeException("高修理车辆策略为空");
            }
            String dir = jobConfig.getCarHighRepairOutputPath();
            Path dirPath = Paths.get(dir);
            if (!Files.exists(dirPath)) {
                Files.createDirectories(dirPath);
                FileUtil.setDefaultDirPermissions(dirPath);
            }
            String filename = "car_high_repair_" + endYearMonth + ".csv";
            Path filePath = Paths.get(dir + filename);
            char delimiter = 1;
            try (OutputStreamWriter osw = new OutputStreamWriter(Files.newOutputStream(filePath),
                    StandardCharsets.UTF_8);
                 CSVPrinter printer = new CSVPrinter(osw,
                         CSVFormat.DEFAULT.builder().setDelimiter(delimiter).build())) {
                // 添加bom头避免excel乱码
                osw.write('\ufeff');
                Map<String, Object> header = list.get(0);
                // 表头
                printer.printRecord(header.keySet());
                for (Map<String, Object> map : list) {
                    printer.printRecord(map.values());
                }
            }
            FileUtil.setDefaultFilePermissions(filePath);
            Files.createDirectories(Paths.get(jobConfig.getTempOutputDir()));
            Files.copy(filePath, Paths.get(jobConfig.getTempOutputDir() + filename));
            long fileSize = Files.size(filePath);
            dataLogPo.setDataSize(fileSize);
            dataLogPo.setDataCount(list.size());
            dataLogPo.setObject(filePath.getFileName().toString());
        } catch (Exception e) {
            log.error(e.toString(), e);
            dataLogPo.setStatus(0);
            dataLogPo.setExceptionInfo(e.toString());
            throw new MyRuntimeException("高修理车辆策略定时任务失败");
        } finally {
            CompletableFuture.runAsync(() -> dataLogService.saveBatch(dataLogPoList));
        }
    }
}
