package com.nokia.finance.tasks.jobs.house.fcdp;

import com.nokia.finance.tasks.common.exception.MyRuntimeException;
import com.nokia.finance.tasks.config.JobConfig;
import com.nokia.finance.tasks.dao.house.HouseFcdpDao;
import com.nokia.finance.tasks.enums.DataStageEnum;
import com.nokia.finance.tasks.pojo.po.common.DataLogPo;
import com.nokia.finance.tasks.service.common.DataLogServiceImpl;
import com.nokia.finance.tasks.utils.FileUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.csv.CSVFormat;
import org.apache.commons.csv.CSVPrinter;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.io.OutputStreamWriter;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.CompletableFuture;

/**
 * 房产大屏建筑和土地数量统计定时任务
 */
@Slf4j
@Service
public class HouseBuildingLandCountJob {
    private final JobConfig jobConfig;
    private final HouseFcdpDao houseFcdpDao;
    private final DataLogServiceImpl dataLogService;

    HouseBuildingLandCountJob(JobConfig jobConfig, HouseFcdpDao houseFcdpDao, DataLogServiceImpl dataLogService) {
        this.jobConfig = jobConfig;
        this.houseFcdpDao = houseFcdpDao;
        this.dataLogService = dataLogService;
    }
    /**
     * 执行任务
     */
    @Scheduled(cron = "0 2 0 12 * ?")
    @Transactional(timeout = 60, rollbackFor = Exception.class)
    public void runJob() {
        log.info("执行房产大屏建筑和土地数量统计定时任务");
        List<DataLogPo> dataLogPoList = new ArrayList<>();
        DataLogPo dataLogPo = new DataLogPo();
        dataLogPo.setDataName("房产大屏建筑和土地数量统计");
        dataLogPo.setStage(DataStageEnum.JG.value);
        dataLogPo.setOperationName("生成房产大屏建筑和土地数量统计");
        dataLogPoList.add(dataLogPo);
        try {
            LocalDate localDateNow = LocalDate.now();
            LocalDate localDateEnd = localDateNow.minusMonths(1);
            Integer endYearMonth = Integer.valueOf(localDateEnd.format(DateTimeFormatter.ofPattern("yyyyMM")));
            int update1 = houseFcdpDao.insertHouseBuildingLandCount(endYearMonth);
            if (update1 == 0) {
                throw new MyRuntimeException("房产大屏建筑和土地数量统计定时任务失败");
            }
            List<LinkedHashMap<String, Object>> list = houseFcdpDao.getHouseBuildingLandCount(endYearMonth);
            String dir = jobConfig.getHouseBuildingLandCountOutputPath();
            Path dirPath = Paths.get(dir);
            if (!Files.exists(dirPath)) {
                Files.createDirectories(dirPath);
                FileUtil.setDefaultDirPermissions(dirPath);
            }
            String filename = "building_land_count_" + endYearMonth + ".csv";
            Path filePath = Paths.get(dir + filename);
            char delimiter = 1;
            try (OutputStreamWriter osw = new OutputStreamWriter(Files.newOutputStream(filePath),
                    StandardCharsets.UTF_8);
                 CSVPrinter printer = new CSVPrinter(osw,
                         CSVFormat.DEFAULT.builder().setDelimiter(delimiter).build())) {
                if (CollectionUtils.isEmpty(list)) {
                    return;
                }
                // 添加bom头避免excel乱码
                osw.write('\ufeff');
                Map<String, Object> header = list.get(0);
                // 表头
                printer.printRecord(header.keySet());
                for (Map<String, Object> map : list) {
                    printer.printRecord(map.values());
                }
            }
            FileUtil.setDefaultFilePermissions(filePath);
            Files.createDirectories(Paths.get(jobConfig.getTempOutputDir()));
            Files.copy(filePath, Paths.get(jobConfig.getTempOutputDir() + filename));
            long fileSize = Files.size(filePath);
            dataLogPo.setDataSize(fileSize);
            dataLogPo.setDataCount(list.size());
            dataLogPo.setObject(filePath.getFileName().toString());
        } catch (Exception e) {
            log.error(e.toString(), e);
            dataLogPo.setStatus(0);
            dataLogPo.setExceptionInfo(e.toString());
            throw new MyRuntimeException("房产大屏建筑和土地数量统计定时任务失败");
        } finally {
            CompletableFuture.runAsync(() -> dataLogService.saveBatch(dataLogPoList));
        }
    }
}
